<?php
if (!defined('ABSPATH')) {
    exit;
}

class SchematicAdminSettings {

    public function __construct() {
        add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
        add_action('admin_menu', array($this, 'menu'), 5);

        add_action('admin_init', array($this, 'checkForUpdates'));
        add_action('admin_notices', array($this, 'notice'));

        add_action('wp_ajax_schematic_api_key_hidden', array($this, 'api_key_hidden'));
        add_action('wp_ajax_schematic_hidden_unhide', array($this, 'hidden_unhide'));
        add_action('wp_ajax_schematic_deregister', array($this, 'deregister'));
        add_action('wp_loaded', array($this, 'process_form'));
    }

    public function deregister() {
        if (current_user_can('administrator')) {
            delete_option('_schematic_d_l');
            delete_option('_schematic_api_key');
            delete_option('_schematic_installed');
        }
    }

    public function notice() {
        if (get_option('_schematic_key_error') && isSchematicInstalled()) {
            ?> <div id="schematic-key-error-message" class="error fade"><p>WP Schema Plugin Notice: <?php echo get_option('_schematic_key_error') ?></p></div><?php
            delete_option('_schematic_d_l');
        }
    }

    public function checkForUpdates() {
        if (current_user_can('administrator') && isset($_GET['check_updates']) && $_GET['check_updates'] == 'true') {
            
        }
    }

    public function admin_scripts() {
        global $schematic_file;

        if (isset($_GET['page']) && ($_GET['page'] === 'wp-schema' || $_GET['page'] === 'wp-schema-installation')) {
            // wp_enqueue_style('schematic-admin', plugins_url('/css/schematic-admin.css', $schematic_file), array(), WPSCHEMA_VERSION);
        }
    }

    function menu() {
        if (isSchematicInstalled()) {
            global $schematicIcon;

            add_menu_page('Schema', 'Schema', 'administrator', 'wp-schema', array($this, 'schematic_settings'), $schematicIcon, 80);
        }
    }

    function getCheckedSchematicPageChosen($id) {
        if (isSchematicPageChosen($id)) {
            echo(' checked="checked" ');
        }
    }

    function sortById($a, $b) {
        if ($a['groupId'] === 'homepage') {
            return 0;
        } elseif ($b['groupId'] === 'homepage') {
            return 1;
        } else {
            return strcmp($a['label'], $b['label']);
        }
    }

    function process_form() {
        if (isset($_POST['savechanges']) && current_user_can('administrator')) {
            if (isset($_POST['_schematic_options'])):
                $existingKey = get_option('_schematic_api_key');
                $key = sanitize_text_field($_POST['_schematic_api_key']);
                if ($existingKey !== $key) {
                    update_option('_schematic_api_key', $key);
                    schematic_fetch_new_api_data();

                    wp_redirect('admin.php?page=wp-schema');
                      exit();
                }
              
            endif;
        }
    }

    function schematic_settings() {
        if (isset($_POST['savechanges']) && current_user_can('administrator')) {
            check_admin_referer('update-schematic-settings');


            if (isset($_POST['_schematic_options'])) {
                $options = $_POST['_schematic_options'];

                update_option('_schematic_options', $options);
            } else {
                delete_option('_schematic_options');
            }

            if ($_POST['_schematic_enabled_pages']) {
                $pages = json_decode(stripslashes($_POST['_schematic_enabled_pages']), true);

                if ($pages && count($pages) > 1) {
                    usort($pages, array($this, 'sortById'));
                }

                update_option('_schematic_enabled_pages', $pages);
            } else {
                delete_option('_schematic_enabled_pages');
            }
            ?>

            <div id="message" class="updated fade"><p>Settings successfully updated.</p></div>
            <?php
        }



        if (!isset($_POST['savechanges']) && isset($_GET['installed']) && $_GET['installed'] == 'true') {
            $notice = get_option('_schematic_installation_complete_notification');

            if (!$notice) {
                ?>
                <div id="message" class="updated fade">
                    <p>Congratulations! Schematic successfully installed. </p>

                </div><?php
                update_option('_schematic_installation_complete_notification', 1);
            }
            ?>

            <?php
        }

        $array = getSchematicPages();
        ?>

        <input id="wpSchemaInstallationUrl" type="hidden" value="<?php echo(admin_url('admin.php?page=wp-schema-installation')) ?>" />         

        <div class="wrap">
            <h2>WP Schema Plugin Settings</h2>
            <div class="postbox">
                <div id="schematic-enabled-pages">
                    <?php
                    $nonce = wp_create_nonce("schematic-nonce");

                    foreach (getSchematicChosenPages() as $chosen) {

                        if (!getSchematicPageByType($chosen['objectType'])) {
                            continue;
                        }
                        ?> <div data-object-type="<?php echo($chosen['objectType']); ?>" data-group-id="<?php echo($chosen['groupId']); ?>" class="enabled-page"><?php getSchematicChosenPageLabelByType($chosen['objectType'], true); ?></div><?php
                    }
                    ?>

                </div>

                <div class="inside">
                    <form novalidate="novalidate" id="schematic-settings-form" data-nonce="<?php echo($nonce); ?>" action="<?php admin_url('admin.php?page=schematic') ?>" method="post">
                        <?php wp_nonce_field('update-schematic-settings') ?>

                        <div class="alignleft" id="schematic-settings">
                            <table class="form-table">

                                <tr valign="top">
                                    <th scope="row"> API Key</th>
                                    <td><input id="_schematic_api_key"

                                               data-errormessage-value-missing="Please enter your Schematic API Key"
                                               required="required"
                                               class="regular-text validate[required] <?php if (is_schematic_api_key_hidden()) { ?> password-field <?php } ?>"
                                               value="<?php echo get_option('_schematic_api_key'); ?>" name="_schematic_api_key"



                                               <?php if (!is_schematic_api_key_hidden()) { ?> type="text" <?php } else { ?> type="password" <?php } ?>
                                               value="<?php echo get_option('_schematic_api_key'); ?>" 

                                               />

                                        <span data-keydata="1" class="key-actions-schematic <?php if (is_schematic_api_key_hidden()) { ?>hidden <?php } ?>">hide</span> 
                                        <span data-keydata="0" class="key-actions-schematic <?php if (!is_schematic_api_key_hidden()) { ?>hidden <?php } ?>">show</span> 


                                        <br /> Your WP Schema Plugin API Key


                                        <b  <?php if (!is_schematic_api_key_hidden()) { ?> class="hidden" <?php } ?>  id="hidden-for-all">is 
                                            hidden for all users</b>

                                    </td>
                                </tr>
                                <tr>
                                    <th scope="row">Enabled Pages</th> 
                                    <td>

                                        <div id="filterSidebars" class="fl">
                                            <div class="fl likeDropdownHolder">
                                                <div class="likeDropdown rc3">
                                                    <div>
                                                        <span>Enabled Pages</span>

                                                    </div>
                                                </div>

                                                <div class="hidden likeDropdownOptions">
                                                    <div class="padding">

                                                        <?php foreach ($array as $objectType => $arr) {
                                                            ?>
                                                            <div  class="mainItem <?php if (isSchematicPageChosen($objectType)) { ?>active<?php } ?>">
                                                                <div class="importantLabel">
                                                                    <span class="label"><?php echo($arr['label']); ?></span>
                                                                    <?php if ($arr['children']) { ?>
                                                                        <span title="Open Submenu" class=" "><i class="icon-options"></i></span>
                                                                    <?php } else { ?>

                                                                        <span style="float:right; margin-right: 15px;">
                                                                            <div class="toggles enableDisableKey">
                                                                                <input <?php $this->getCheckedSchematicPageChosen($objectType); ?> 
                                                                                    data-object-type="<?php echo($objectType); ?>" 
                                                                                    data-group-id="<?php echo($arr['groupId']); ?>" 
                                                                                    data-label="<?php echo($arr['label']); ?>" type="checkbox" name="<?php echo($arr['groupId']); ?>" id="checkbox-<?php echo($arr['groupId']); ?>" class="ios-toggle" />
                                                                                <label for="checkbox-<?php echo($arr['groupId']); ?>" class="checkbox-label" data-off="off" data-on="on"></label>
                                                                            </div>
                                                                        </span>

                                                                    <?php } ?>
                                                                </div>
                                                            </div>

                                                            <?php if ($arr['children']) { ?>
                                                                <div class="items hidden">
                                                                    <?php foreach ($arr['children'] as $children) { ?>
                                                                        <div class="item <?php if (isSchematicPageChosen($children['objectType'])) { ?>active<?php } ?>">
                                                                            <span class="label"><?php echo($children['label']); ?></span>
                                                                            <span class="ios-style-buttons" style="float:right; margin-right: 15px;">
                                                                                <div class="toggles enableDisableKey">
                                                                                    <input data-object-type="<?php echo($children['objectType']); ?>" <?php $this->getCheckedSchematicPageChosen($children['objectType']); ?> 
                                                                                           data-group-id="<?php echo($objectType); ?>" data-label="<?php echo($children['label']); ?>" type="checkbox" name="<?php echo($children['objectType']); ?>" 
                                                                                           id="checkbox-<?php echo($children['objectType']); ?>" class="ios-toggle" />
                                                                                    <label for="checkbox-<?php echo($children['objectType']); ?>" class="checkbox-label" data-off="off" data-on="on"></label>
                                                                                </div>
                                                                            </span>
                                                                        </div>
                                                                    <?php } ?>
                                                                </div>
                                                            <?php } ?>
                                                        <?php } ?>
                                                    </div>
                                                </div>
                                            </div>
                                        </div>

                                        <input type="hidden" value='<?php echo(json_encode(getSchematicChosenPages())); ?>' id="schematic-pages" name="_schematic_enabled_pages" />
                                        Choose which pages you would like to have schema feature

                                    </td>
                                </tr>

                                <tr>
                                    <th>
                                        Options
                                    </th>
                                    <td>
                                        <?php
                                        $view = new SchematicOptionsView();
                                        $view->renderView();
                                        ?>
                                    </td>
                                </tr>

                            </table>

                            <p class="submit"><input class="button button-primary" type="submit" name="savechanges" value="Save Settings" /></p>
                        </div>
                        <div class="clear"></div>

                    </form>
                </div>
            </div>
        </div>


        <?php
        getSchematicEditButtonHtml(null, null, true);
    }

    public function api_key_hidden() {
        if (current_user_can('administrator')) {

            if (schematic_hidden_by_check()) {
                $action = $_POST['keydata'];
                update_option('_schematic_api_key_hidden_by', get_current_user_id());

                update_option('_schematic_api_key_hidden', $action);
            }

            wp_send_json_success();
        }
    }

    public function hidden_unhide() {
        if (current_user_can('administrator')) {
            delete_option('_schematic_api_key_hidden');
            delete_option('_schematic_api_key_hidden_by');

            wp_send_json_success();
        }

        wp_send_json_error();
    }

}

new SchematicAdminSettings();
